import { FastifyRequest, FastifyReply } from 'fastify';
import { User } from '../../models/User';
import bcrypt from 'bcryptjs';
import jwt from 'jsonwebtoken';

const JWT_SECRET = process.env.JWT_SECRET || 'yourSuperSecretKey';

const loginSchema = {
  type: 'object',
  properties: {
    email: { type: 'string' },
    password: { type: 'string' },
  },
  required: ['email', 'password'],
};

export const loginRouteOptions = {
  schema: {
    body: loginSchema, 
  },
};

// Login function
export const login = async (req: FastifyRequest, reply: FastifyReply) => {
  const { email, password } = req.body as { email: string; password: string };

  try {
    console.log('Attempting login for:', email);
    
    const user = await User.findOne({ where: { email } });
    
    if (!user) {
      console.log('User not found');
      return reply.code(401).send({ message: 'Invalid credentials' });
    }

    console.log('User found:', user.email);
    
    // Add this check to ensure user.password exists
    if (!user.password) {
      console.log('No password set for user');
      return reply.code(401).send({ message: 'Invalid credentials' });
    }

    console.log('Comparing passwords...');
    const isMatch = await bcrypt.compare(password, user.password);
    console.log('Password match:', isMatch);

    if (!isMatch) {
      return reply.code(401).send({ message: 'Invalid credentials' });
    }

    const token = jwt.sign(
      { id: user.id, email: user.email, role: user.role },
      JWT_SECRET,
      { expiresIn: '1h' }
    );

    return reply.send({ 
      message: 'Login successful', 
      token,
      user: {
        id: user.id,
        email: user.email,
        role: user.role
      }
    });
  } catch (error) {
    console.error('Login error:', error);
    return reply.code(500).send({ 
      message: 'Internal server error',
      error: error instanceof Error ? error.message : 'Unknown error'
    });
  }
};

// Register function
interface RegisterRequestBody {
  email: string;
  password: string;
  role?: string;
}

export const register = async (
  req: FastifyRequest<{ Body: RegisterRequestBody }>,
  reply: FastifyReply
) => {
  const { email, password, role } = req.body;

  try {
    const existingUser = await User.findOne({ where: { email } });
    if (existingUser) {
      return reply.status(400).send({ message: 'Email is already registered' });
    }

    const hashedPassword = await bcrypt.hash(password, 10);

    const newUser = await User.create({
      email,
      password: hashedPassword,
      role: role || 'user',
    });

    return reply.status(201).send({ message: 'User registered successfully', userId: newUser.id });
  } catch (error) {
    console.error('Register error:', error);
    return reply.status(500).send({ message: 'Internal server error' });
  }
};